@@ -61,17 +61,10 @@ total_i(VALUE v, void *ptr)
 {
     struct total_data *data = (struct total_data *)ptr;
 
-    switch (BUILTIN_TYPE(v)) {
-      case T_NONE:
-      case T_IMEMO:
-      case T_ICLASS:
-      case T_NODE:
-      case T_ZOMBIE:
-          return;
-      default:
-          if (data->klass == 0 || rb_obj_is_kind_of(v, data->klass)) {
-              data->total += rb_obj_memsize_of(v);
-          }
+    if (!rb_objspace_internal_object_p(v)) {
+        if (data->klass == 0 || rb_obj_is_kind_of(v, data->klass)) {
+            data->total += rb_obj_memsize_of(v);
+        }
     }
 }
 
@@ -147,7 +140,7 @@ memsize_of_all_m(int argc, VALUE *argv, VALUE self)
     struct total_data data = {0, 0};
 
     if (argc > 0) {
-	rb_scan_args(argc, argv, "01", &data.klass);
+        rb_scan_args(argc, argv, "01", &data.klass);
     }
 
     each_object_with_flags(total_i, &data);
@@ -197,33 +190,33 @@ type2sym(enum ruby_value_type i)
     VALUE type;
     switch (i) {
 #define CASE_TYPE(t) case t: type = ID2SYM(rb_intern(#t)); break;
-	CASE_TYPE(T_NONE);
-	CASE_TYPE(T_OBJECT);
-	CASE_TYPE(T_CLASS);
-	CASE_TYPE(T_MODULE);
-	CASE_TYPE(T_FLOAT);
-	CASE_TYPE(T_STRING);
-	CASE_TYPE(T_REGEXP);
-	CASE_TYPE(T_ARRAY);
-	CASE_TYPE(T_HASH);
-	CASE_TYPE(T_STRUCT);
-	CASE_TYPE(T_BIGNUM);
-	CASE_TYPE(T_FILE);
-	CASE_TYPE(T_DATA);
-	CASE_TYPE(T_MATCH);
-	CASE_TYPE(T_COMPLEX);
-	CASE_TYPE(T_RATIONAL);
-	CASE_TYPE(T_NIL);
-	CASE_TYPE(T_TRUE);
-	CASE_TYPE(T_FALSE);
-	CASE_TYPE(T_SYMBOL);
-	CASE_TYPE(T_FIXNUM);
-	CASE_TYPE(T_UNDEF);
-	CASE_TYPE(T_IMEMO);
-	CASE_TYPE(T_NODE);
-	CASE_TYPE(T_ICLASS);
+        CASE_TYPE(T_NONE);
+        CASE_TYPE(T_OBJECT);
+        CASE_TYPE(T_CLASS);
+        CASE_TYPE(T_MODULE);
+        CASE_TYPE(T_FLOAT);
+        CASE_TYPE(T_STRING);
+        CASE_TYPE(T_REGEXP);
+        CASE_TYPE(T_ARRAY);
+        CASE_TYPE(T_HASH);
+        CASE_TYPE(T_STRUCT);
+        CASE_TYPE(T_BIGNUM);
+        CASE_TYPE(T_FILE);
+        CASE_TYPE(T_DATA);
+        CASE_TYPE(T_MATCH);
+        CASE_TYPE(T_COMPLEX);
+        CASE_TYPE(T_RATIONAL);
+        CASE_TYPE(T_NIL);
+        CASE_TYPE(T_TRUE);
+        CASE_TYPE(T_FALSE);
+        CASE_TYPE(T_SYMBOL);
+        CASE_TYPE(T_FIXNUM);
+        CASE_TYPE(T_UNDEF);
+        CASE_TYPE(T_IMEMO);
+        CASE_TYPE(T_NODE);
+        CASE_TYPE(T_ICLASS);
         CASE_TYPE(T_MOVED);
-	CASE_TYPE(T_ZOMBIE);
+        CASE_TYPE(T_ZOMBIE);
 #undef CASE_TYPE
       default: rb_bug("type2sym: unknown type (%d)", i);
     }
@@ -262,17 +255,17 @@ count_objects_size(int argc, VALUE *argv, VALUE os)
     VALUE hash = setup_hash(argc, argv);
 
     for (i = 0; i <= T_MASK; i++) {
-	counts[i] = 0;
+        counts[i] = 0;
     }
 
     each_object_with_flags(cos_i, &counts[0]);
 
     for (i = 0; i <= T_MASK; i++) {
-	if (counts[i]) {
-	    VALUE type = type2sym(i);
-	    total += counts[i];
-	    rb_hash_aset(hash, type, SIZET2NUM(counts[i]));
-	}
+        if (counts[i]) {
+            VALUE type = type2sym(i);
+            total += counts[i];
+            rb_hash_aset(hash, type, SIZET2NUM(counts[i]));
+        }
     }
     rb_hash_aset(hash, ID2SYM(rb_intern("TOTAL")), SIZET2NUM(total));
     return hash;
@@ -386,128 +379,127 @@ count_nodes(int argc, VALUE *argv, VALUE os)
     VALUE hash = setup_hash(argc, argv);
 
     for (i = 0; i <= NODE_LAST; i++) {
-	nodes[i] = 0;
+        nodes[i] = 0;
     }
 
     each_object_with_flags(cn_i, &nodes[0]);
 
     for (i=0; i<NODE_LAST; i++) {
-	if (nodes[i] != 0) {
-	    VALUE node;
-	    switch (i) {
+        if (nodes[i] != 0) {
+            VALUE node;
+            switch (i) {
 #define COUNT_NODE(n) case n: node = ID2SYM(rb_intern(#n)); goto set
-		COUNT_NODE(NODE_SCOPE);
-		COUNT_NODE(NODE_BLOCK);
-		COUNT_NODE(NODE_IF);
-		COUNT_NODE(NODE_UNLESS);
-		COUNT_NODE(NODE_CASE);
-		COUNT_NODE(NODE_CASE2);
-		COUNT_NODE(NODE_CASE3);
-		COUNT_NODE(NODE_WHEN);
-		COUNT_NODE(NODE_IN);
-		COUNT_NODE(NODE_WHILE);
-		COUNT_NODE(NODE_UNTIL);
-		COUNT_NODE(NODE_ITER);
-		COUNT_NODE(NODE_FOR);
-		COUNT_NODE(NODE_FOR_MASGN);
-		COUNT_NODE(NODE_BREAK);
-		COUNT_NODE(NODE_NEXT);
-		COUNT_NODE(NODE_REDO);
-		COUNT_NODE(NODE_RETRY);
-		COUNT_NODE(NODE_BEGIN);
-		COUNT_NODE(NODE_RESCUE);
-		COUNT_NODE(NODE_RESBODY);
-		COUNT_NODE(NODE_ENSURE);
-		COUNT_NODE(NODE_AND);
-		COUNT_NODE(NODE_OR);
-		COUNT_NODE(NODE_MASGN);
-		COUNT_NODE(NODE_LASGN);
-		COUNT_NODE(NODE_DASGN);
-		COUNT_NODE(NODE_DASGN_CURR);
-		COUNT_NODE(NODE_GASGN);
-		COUNT_NODE(NODE_IASGN);
-		COUNT_NODE(NODE_CDECL);
-		COUNT_NODE(NODE_CVASGN);
-		COUNT_NODE(NODE_OP_ASGN1);
-		COUNT_NODE(NODE_OP_ASGN2);
-		COUNT_NODE(NODE_OP_ASGN_AND);
-		COUNT_NODE(NODE_OP_ASGN_OR);
-		COUNT_NODE(NODE_OP_CDECL);
-		COUNT_NODE(NODE_CALL);
-		COUNT_NODE(NODE_OPCALL);
-		COUNT_NODE(NODE_FCALL);
-		COUNT_NODE(NODE_VCALL);
-		COUNT_NODE(NODE_QCALL);
-		COUNT_NODE(NODE_SUPER);
-		COUNT_NODE(NODE_ZSUPER);
-		COUNT_NODE(NODE_LIST);
-		COUNT_NODE(NODE_ZLIST);
-		COUNT_NODE(NODE_VALUES);
-		COUNT_NODE(NODE_HASH);
-		COUNT_NODE(NODE_RETURN);
-		COUNT_NODE(NODE_YIELD);
-		COUNT_NODE(NODE_LVAR);
-		COUNT_NODE(NODE_DVAR);
-		COUNT_NODE(NODE_GVAR);
-		COUNT_NODE(NODE_IVAR);
-		COUNT_NODE(NODE_CONST);
-		COUNT_NODE(NODE_CVAR);
-		COUNT_NODE(NODE_NTH_REF);
-		COUNT_NODE(NODE_BACK_REF);
-		COUNT_NODE(NODE_MATCH);
-		COUNT_NODE(NODE_MATCH2);
-		COUNT_NODE(NODE_MATCH3);
-		COUNT_NODE(NODE_LIT);
-		COUNT_NODE(NODE_STR);
-		COUNT_NODE(NODE_DSTR);
-		COUNT_NODE(NODE_XSTR);
-		COUNT_NODE(NODE_DXSTR);
-		COUNT_NODE(NODE_EVSTR);
-		COUNT_NODE(NODE_DREGX);
-		COUNT_NODE(NODE_ONCE);
-		COUNT_NODE(NODE_ARGS);
-		COUNT_NODE(NODE_ARGS_AUX);
-		COUNT_NODE(NODE_OPT_ARG);
-		COUNT_NODE(NODE_KW_ARG);
-		COUNT_NODE(NODE_POSTARG);
-		COUNT_NODE(NODE_ARGSCAT);
-		COUNT_NODE(NODE_ARGSPUSH);
-		COUNT_NODE(NODE_SPLAT);
-		COUNT_NODE(NODE_BLOCK_PASS);
-		COUNT_NODE(NODE_DEFN);
-		COUNT_NODE(NODE_DEFS);
-		COUNT_NODE(NODE_ALIAS);
-		COUNT_NODE(NODE_VALIAS);
-		COUNT_NODE(NODE_UNDEF);
-		COUNT_NODE(NODE_CLASS);
-		COUNT_NODE(NODE_MODULE);
-		COUNT_NODE(NODE_SCLASS);
-		COUNT_NODE(NODE_COLON2);
-		COUNT_NODE(NODE_COLON3);
-		COUNT_NODE(NODE_DOT2);
-		COUNT_NODE(NODE_DOT3);
-		COUNT_NODE(NODE_FLIP2);
-		COUNT_NODE(NODE_FLIP3);
-		COUNT_NODE(NODE_SELF);
-		COUNT_NODE(NODE_NIL);
-		COUNT_NODE(NODE_TRUE);
-		COUNT_NODE(NODE_FALSE);
-		COUNT_NODE(NODE_ERRINFO);
-		COUNT_NODE(NODE_DEFINED);
-		COUNT_NODE(NODE_POSTEXE);
-		COUNT_NODE(NODE_DSYM);
-		COUNT_NODE(NODE_ATTRASGN);
-		COUNT_NODE(NODE_LAMBDA);
-		COUNT_NODE(NODE_ARYPTN);
-		COUNT_NODE(NODE_FNDPTN);
-		COUNT_NODE(NODE_HSHPTN);
+                COUNT_NODE(NODE_SCOPE);
+                COUNT_NODE(NODE_BLOCK);
+                COUNT_NODE(NODE_IF);
+                COUNT_NODE(NODE_UNLESS);
+                COUNT_NODE(NODE_CASE);
+                COUNT_NODE(NODE_CASE2);
+                COUNT_NODE(NODE_CASE3);
+                COUNT_NODE(NODE_WHEN);
+                COUNT_NODE(NODE_IN);
+                COUNT_NODE(NODE_WHILE);
+                COUNT_NODE(NODE_UNTIL);
+                COUNT_NODE(NODE_ITER);
+                COUNT_NODE(NODE_FOR);
+                COUNT_NODE(NODE_FOR_MASGN);
+                COUNT_NODE(NODE_BREAK);
+                COUNT_NODE(NODE_NEXT);
+                COUNT_NODE(NODE_REDO);
+                COUNT_NODE(NODE_RETRY);
+                COUNT_NODE(NODE_BEGIN);
+                COUNT_NODE(NODE_RESCUE);
+                COUNT_NODE(NODE_RESBODY);
+                COUNT_NODE(NODE_ENSURE);
+                COUNT_NODE(NODE_AND);
+                COUNT_NODE(NODE_OR);
+                COUNT_NODE(NODE_MASGN);
+                COUNT_NODE(NODE_LASGN);
+                COUNT_NODE(NODE_DASGN);
+                COUNT_NODE(NODE_GASGN);
+                COUNT_NODE(NODE_IASGN);
+                COUNT_NODE(NODE_CDECL);
+                COUNT_NODE(NODE_CVASGN);
+                COUNT_NODE(NODE_OP_ASGN1);
+                COUNT_NODE(NODE_OP_ASGN2);
+                COUNT_NODE(NODE_OP_ASGN_AND);
+                COUNT_NODE(NODE_OP_ASGN_OR);
+                COUNT_NODE(NODE_OP_CDECL);
+                COUNT_NODE(NODE_CALL);
+                COUNT_NODE(NODE_OPCALL);
+                COUNT_NODE(NODE_FCALL);
+                COUNT_NODE(NODE_VCALL);
+                COUNT_NODE(NODE_QCALL);
+                COUNT_NODE(NODE_SUPER);
+                COUNT_NODE(NODE_ZSUPER);
+                COUNT_NODE(NODE_LIST);
+                COUNT_NODE(NODE_ZLIST);
+                COUNT_NODE(NODE_VALUES);
+                COUNT_NODE(NODE_HASH);
+                COUNT_NODE(NODE_RETURN);
+                COUNT_NODE(NODE_YIELD);
+                COUNT_NODE(NODE_LVAR);
+                COUNT_NODE(NODE_DVAR);
+                COUNT_NODE(NODE_GVAR);
+                COUNT_NODE(NODE_IVAR);
+                COUNT_NODE(NODE_CONST);
+                COUNT_NODE(NODE_CVAR);
+                COUNT_NODE(NODE_NTH_REF);
+                COUNT_NODE(NODE_BACK_REF);
+                COUNT_NODE(NODE_MATCH);
+                COUNT_NODE(NODE_MATCH2);
+                COUNT_NODE(NODE_MATCH3);
+                COUNT_NODE(NODE_LIT);
+                COUNT_NODE(NODE_STR);
+                COUNT_NODE(NODE_DSTR);
+                COUNT_NODE(NODE_XSTR);
+                COUNT_NODE(NODE_DXSTR);
+                COUNT_NODE(NODE_EVSTR);
+                COUNT_NODE(NODE_DREGX);
+                COUNT_NODE(NODE_ONCE);
+                COUNT_NODE(NODE_ARGS);
+                COUNT_NODE(NODE_ARGS_AUX);
+                COUNT_NODE(NODE_OPT_ARG);
+                COUNT_NODE(NODE_KW_ARG);
+                COUNT_NODE(NODE_POSTARG);
+                COUNT_NODE(NODE_ARGSCAT);
+                COUNT_NODE(NODE_ARGSPUSH);
+                COUNT_NODE(NODE_SPLAT);
+                COUNT_NODE(NODE_BLOCK_PASS);
+                COUNT_NODE(NODE_DEFN);
+                COUNT_NODE(NODE_DEFS);
+                COUNT_NODE(NODE_ALIAS);
+                COUNT_NODE(NODE_VALIAS);
+                COUNT_NODE(NODE_UNDEF);
+                COUNT_NODE(NODE_CLASS);
+                COUNT_NODE(NODE_MODULE);
+                COUNT_NODE(NODE_SCLASS);
+                COUNT_NODE(NODE_COLON2);
+                COUNT_NODE(NODE_COLON3);
+                COUNT_NODE(NODE_DOT2);
+                COUNT_NODE(NODE_DOT3);
+                COUNT_NODE(NODE_FLIP2);
+                COUNT_NODE(NODE_FLIP3);
+                COUNT_NODE(NODE_SELF);
+                COUNT_NODE(NODE_NIL);
+                COUNT_NODE(NODE_TRUE);
+                COUNT_NODE(NODE_FALSE);
+                COUNT_NODE(NODE_ERRINFO);
+                COUNT_NODE(NODE_DEFINED);
+                COUNT_NODE(NODE_POSTEXE);
+                COUNT_NODE(NODE_DSYM);
+                COUNT_NODE(NODE_ATTRASGN);
+                COUNT_NODE(NODE_LAMBDA);
+                COUNT_NODE(NODE_ARYPTN);
+                COUNT_NODE(NODE_FNDPTN);
+                COUNT_NODE(NODE_HSHPTN);
 #undef COUNT_NODE
-	      case NODE_LAST: break;
-	    }
-	    UNREACHABLE;
-	  set:
-	    rb_hash_aset(hash, node, SIZET2NUM(nodes[i]));
-	}
+              case NODE_LAST: break;
+            }
+            UNREACHABLE;
+          set:
+            rb_hash_aset(hash, node, SIZET2NUM(nodes[i]));
+        }
     }
     return hash;
 }
@@ -637,19 +629,22 @@ count_imemo_objects(int argc, VALUE *argv, VALUE self)
     VALUE hash = setup_hash(argc, argv);
 
     if (imemo_type_ids[0] == 0) {
-        imemo_type_ids[0] = rb_intern("imemo_env");
-	imemo_type_ids[1] = rb_intern("imemo_cref");
-	imemo_type_ids[2] = rb_intern("imemo_svar");
-	imemo_type_ids[3] = rb_intern("imemo_throw_data");
-	imemo_type_ids[4] = rb_intern("imemo_ifunc");
-	imemo_type_ids[5] = rb_intern("imemo_memo");
-	imemo_type_ids[6] = rb_intern("imemo_ment");
-	imemo_type_ids[7] = rb_intern("imemo_iseq");
-	imemo_type_ids[8] = rb_intern("imemo_tmpbuf");
-        imemo_type_ids[9] = rb_intern("imemo_ast");
-        imemo_type_ids[10] = rb_intern("imemo_parser_strterm");
-        imemo_type_ids[11] = rb_intern("imemo_callinfo");
-        imemo_type_ids[12] = rb_intern("imemo_callcache");
+#define INIT_IMEMO_TYPE_ID(n) (imemo_type_ids[n] = rb_intern_const(#n))
+        INIT_IMEMO_TYPE_ID(imemo_env);
+        INIT_IMEMO_TYPE_ID(imemo_cref);
+        INIT_IMEMO_TYPE_ID(imemo_svar);
+        INIT_IMEMO_TYPE_ID(imemo_throw_data);
+        INIT_IMEMO_TYPE_ID(imemo_ifunc);
+        INIT_IMEMO_TYPE_ID(imemo_memo);
+        INIT_IMEMO_TYPE_ID(imemo_ment);
+        INIT_IMEMO_TYPE_ID(imemo_iseq);
+        INIT_IMEMO_TYPE_ID(imemo_tmpbuf);
+        INIT_IMEMO_TYPE_ID(imemo_ast);
+        INIT_IMEMO_TYPE_ID(imemo_parser_strterm);
+        INIT_IMEMO_TYPE_ID(imemo_callinfo);
+        INIT_IMEMO_TYPE_ID(imemo_callcache);
+        INIT_IMEMO_TYPE_ID(imemo_constcache);
+#undef INIT_IMEMO_TYPE_ID
     }
 
     each_object_with_flags(count_imemo_objects_i, (void *)hash);
@@ -712,7 +707,7 @@ iow_internal_object_id(VALUE self)
 
 struct rof_data {
     VALUE refs;
-    VALUE internals;
+    VALUE values;
 };
 
 static void
@@ -723,11 +718,15 @@ reachable_object_from_i(VALUE obj, void *data_ptr)
     VALUE val = obj;
 
     if (rb_objspace_markable_object_p(obj)) {
-	if (rb_objspace_internal_object_p(obj)) {
-	    val = iow_newobj(obj);
-	    rb_ary_push(data->internals, val);
-	}
-	rb_hash_aset(data->refs, key, val);
+        if (NIL_P(rb_hash_lookup(data->refs, key))) {
+            rb_hash_aset(data->refs, key, Qtrue);
+
+            if (rb_objspace_internal_object_p(obj)) {
+                val = iow_newobj(obj);
+            }
+
+            rb_ary_push(data->values, val);
+        }
     }
 }
 
@@ -785,21 +784,21 @@ static VALUE
 reachable_objects_from(VALUE self, VALUE obj)
 {
     if (rb_objspace_markable_object_p(obj)) {
-	struct rof_data data;
+        struct rof_data data;
 
-	if (rb_typeddata_is_kind_of(obj, &iow_data_type)) {
-	    obj = (VALUE)DATA_PTR(obj);
-	}
+        if (rb_typeddata_is_kind_of(obj, &iow_data_type)) {
+            obj = (VALUE)DATA_PTR(obj);
+        }
 
-	data.refs = rb_ident_hash_new();
-	data.internals = rb_ary_new();
+        data.refs = rb_obj_hide(rb_ident_hash_new());
+        data.values = rb_ary_new();
 
-	rb_objspace_reachable_objects_from(obj, reachable_object_from_i, &data);
+        rb_objspace_reachable_objects_from(obj, reachable_object_from_i, &data);
 
-        return rb_funcall(data.refs, rb_intern("values"), 0);
+        return data.values;
     }
     else {
-	return Qnil;
+        return Qnil;
     }
 }
 
@@ -818,26 +817,26 @@ reachable_object_from_root_i(const char *category, VALUE obj, void *ptr)
     VALUE category_objects;
 
     if (category == data->last_category) {
-	category_str = data->last_category_str;
-	category_objects = data->last_category_objects;
+        category_str = data->last_category_str;
+        category_objects = data->last_category_objects;
     }
     else {
-	data->last_category = category;
-	category_str = data->last_category_str = rb_str_new2(category);
-	category_objects = data->last_category_objects = rb_ident_hash_new();
-	if (!NIL_P(rb_hash_lookup(data->categories, category_str))) {
-	    rb_bug("reachable_object_from_root_i: category should insert at once");
-	}
-	rb_hash_aset(data->categories, category_str, category_objects);
+        data->last_category = category;
+        category_str = data->last_category_str = rb_str_new2(category);
+        category_objects = data->last_category_objects = rb_ident_hash_new();
+        if (!NIL_P(rb_hash_lookup(data->categories, category_str))) {
+            rb_bug("reachable_object_from_root_i: category should insert at once");
+        }
+        rb_hash_aset(data->categories, category_str, category_objects);
     }
 
     if (rb_objspace_markable_object_p(obj) &&
-	obj != data->categories &&
-	obj != data->last_category_objects) {
-	if (rb_objspace_internal_object_p(obj)) {
-	    obj = iow_newobj(obj);
-	}
-	rb_hash_aset(category_objects, obj, obj);
+        obj != data->categories &&
+        obj != data->last_category_objects) {
+        if (rb_objspace_internal_object_p(obj)) {
+            obj = iow_newobj(obj);
+        }
+        rb_hash_aset(category_objects, obj, obj);
     }
 }
 
@@ -873,14 +872,14 @@ static VALUE
 wrap_klass_iow(VALUE klass)
 {
     if (!RTEST(klass)) {
-	return Qnil;
+        return Qnil;
     }
     else if (RB_TYPE_P(klass, T_ICLASS) ||
              CLASS_OF(klass) == Qfalse /* hidden object */) {
-	return iow_newobj(klass);
+        return iow_newobj(klass);
     }
     else {
-	return klass;
+        return klass;
     }
 }
 
@@ -899,7 +898,7 @@ objspace_internal_class_of(VALUE self, VALUE obj)
     VALUE klass;
 
     if (rb_typeddata_is_kind_of(obj, &iow_data_type)) {
-	obj = (VALUE)DATA_PTR(obj);
+        obj = (VALUE)DATA_PTR(obj);
     }
 
     if (RB_TYPE_P(obj, T_IMEMO)) {
@@ -926,17 +925,17 @@ objspace_internal_super_of(VALUE self, VALUE obj)
     VALUE super;
 
     if (rb_typeddata_is_kind_of(obj, &iow_data_type)) {
-	obj = (VALUE)DATA_PTR(obj);
+        obj = (VALUE)DATA_PTR(obj);
     }
 
     switch (OBJ_BUILTIN_TYPE(obj)) {
       case T_MODULE:
       case T_CLASS:
       case T_ICLASS:
-	super = RCLASS_SUPER(obj);
-	break;
+        super = RCLASS_SUPER(obj);
+        break;
       default:
-	rb_raise(rb_eArgError, "class or module is expected");
+        rb_raise(rb_eArgError, "class or module is expected");
     }
 
     return wrap_klass_iow(super);
@@ -995,6 +994,7 @@ Init_objspace(void)
      * You can use the #type method to check the type of the internal object.
      */
     rb_cInternalObjectWrapper = rb_define_class_under(rb_mObjSpace, "InternalObjectWrapper", rb_cObject);
+    rb_undef_alloc_func(rb_cInternalObjectWrapper);
     rb_define_method(rb_cInternalObjectWrapper, "type", iow_type, 0);
     rb_define_method(rb_cInternalObjectWrapper, "inspect", iow_inspect, 0);
     rb_define_method(rb_cInternalObjectWrapper, "internal_object_id", iow_internal_object_id, 0);
