@@ -23,102 +23,33 @@ def IO.pread(*args)
   popen(*args) {|f|f.read}
 end
 
-if RUBY_VERSION < "2.0"
-  class IO
-    @orig_popen = method(:popen)
-
-    if defined?(fork)
-      def self.popen(command, *rest, &block)
-        if command.kind_of?(Hash)
-          env = command
-          command = rest.shift
-        end
-        opts = rest.last
-        if opts.kind_of?(Hash)
-          dir = opts.delete(:chdir)
-          rest.pop if opts.empty?
-          opts.delete(:external_encoding)
-        end
-
-        if block
-          @orig_popen.call("-", *rest) do |f|
-            if f
-              yield(f)
-            else
-              Dir.chdir(dir) if dir
-              ENV.replace(env) if env
-              exec(*command)
-            end
-          end
-        else
-          f = @orig_popen.call("-", *rest)
-          unless f
-            Dir.chdir(dir) if dir
-            ENV.replace(env) if env
-            exec(*command)
-          end
-          f
-        end
-      end
-    else
-      require 'shellwords'
-      def self.popen(command, *rest, &block)
-        if command.kind_of?(Hash)
-          env = command
-          oldenv = ENV.to_hash
-          command = rest.shift
-        end
-        opts = rest.last
-        if opts.kind_of?(Hash)
-          dir = opts.delete(:chdir)
-          rest.pop if opts.empty?
-          opts.delete(:external_encoding)
-        end
-
-        command = command.shelljoin if Array === command
-        Dir.chdir(dir || ".") do
-          ENV.replace(env) if env
-          @orig_popen.call(command, *rest, &block)
-          ENV.replace(oldenv) if oldenv
-        end
-      end
-    end
-  end
-else
-  module DebugPOpen
-    verbose, $VERBOSE = $VERBOSE, nil if RUBY_VERSION < "2.1"
-    refine IO.singleton_class do
-      def popen(*args)
-        VCS::DEBUG_OUT.puts args.inspect if $DEBUG
-        super
-      end
-    end
-  ensure
-    $VERBOSE = verbose unless verbose.nil?
-  end
-  using DebugPOpen
-  module DebugSystem
-    def system(*args)
+module DebugPOpen
+  refine IO.singleton_class do
+    def popen(*args)
       VCS::DEBUG_OUT.puts args.inspect if $DEBUG
-      exception = false
-      opts = Hash.try_convert(args[-1])
-      if RUBY_VERSION >= "2.6"
-        unless opts
-          opts = {}
-          args << opts
-        end
-        exception = opts.fetch(:exception) {opts[:exception] = true}
-      elsif opts
-        exception = opts.delete(:exception) {true}
-        args.pop if opts.empty?
-      end
-      ret = super(*args)
-      raise "Command failed with status (#$?): #{args[0]}" if exception and !ret
-      ret
+      super
     end
   end
-  module Kernel
-    prepend(DebugSystem)
+end
+using DebugPOpen
+module DebugSystem
+  def system(*args)
+    VCS::DEBUG_OUT.puts args.inspect if $DEBUG
+    exception = false
+    opts = Hash.try_convert(args[-1])
+    if RUBY_VERSION >= "2.6"
+      unless opts
+        opts = {}
+        args << opts
+      end
+      exception = opts.fetch(:exception) {opts[:exception] = true}
+    elsif opts
+      exception = opts.delete(:exception) {true}
+      args.pop if opts.empty?
+    end
+    ret = super(*args)
+    raise "Command failed with status (#$?): #{args[0]}" if exception and !ret
+    ret
   end
 end
 
@@ -131,7 +62,8 @@ class VCS
     @@dirs << [dir, self, pred]
   end
 
-  def self.detect(path = '.', options = {}, parser = nil)
+  def self.detect(path = '.', options = {}, parser = nil, **opts)
+    options.update(opts)
     uplevel_limit = options.fetch(:uplevel_limit, 0)
     curr = path
     begin
@@ -200,7 +132,7 @@ class VCS
     end
     last, changed, modified, *rest = (
       begin
-        if NullDevice
+        if NullDevice and !debug?
           save_stderr = STDERR.dup
           STDERR.reopen NullDevice, 'w'
         end
@@ -272,6 +204,34 @@ class VCS
     revision_handler(rev).short_revision(rev)
   end
 
+  def revision_header(last, modified = nil, branch = nil, title = nil, limit: 20)
+    short = short_revision(last)
+    if /[^\x00-\x7f]/ =~ title and title.respond_to?(:force_encoding)
+      title = title.dup.force_encoding("US-ASCII")
+    end
+    code = [
+      "#define RUBY_REVISION #{short.inspect}",
+    ]
+    unless short == last
+      code << "#define RUBY_FULL_REVISION #{last.inspect}"
+    end
+    if branch
+      e = '..'
+      name = branch.sub(/\A(.{#{limit-e.size}}).{#{e.size+1},}/o) {$1+e}
+      name = name.dump.sub(/\\#/, '#')
+      code << "#define RUBY_BRANCH_NAME #{name}"
+    end
+    if title
+      title = title.dump.sub(/\\#/, '#')
+      code << "#define RUBY_LAST_COMMIT_TITLE #{title}"
+    end
+    if modified
+      t = modified.utc
+      code << t.strftime('#define RUBY_RELEASE_DATETIME "%FT%TZ"')
+    end
+    code
+  end
+
   class SVN < self
     register(".svn")
     COMMAND = ENV['SVN'] || 'svn'
@@ -407,13 +367,11 @@ class VCS
       rev.to_i if rev
     end
 
-    def export_changelog(url, from, to, path)
+    def export_changelog(url = '.', from = nil, to = nil, _path = nil, path: _path)
       range = [to || 'HEAD', (from ? from+1 : branch_beginning(url))].compact.join(':')
       IO.popen({'TZ' => 'JST-9', 'LANG' => 'C', 'LC_ALL' => 'C'},
                %W"#{COMMAND} log -r#{range} #{url}") do |r|
-        open(path, 'w') do |w|
-          IO.copy_stream(r, w)
-        end
+        IO.copy_stream(r, path)
       end
     end
 
@@ -530,10 +488,13 @@ class VCS
     end
 
     def without_gitconfig
-      home = ENV.delete('HOME')
+      envs = %w'HOME XDG_CONFIG_HOME GIT_SYSTEM_CONFIG GIT_CONFIG_SYSTEM'.each_with_object({}) do |v, h|
+        h[v] = ENV.delete(v)
+        ENV[v] = NullDevice if v.start_with?('GIT_')
+      end
       yield
     ensure
-      ENV['HOME'] = home if home
+      ENV.update(envs)
     end
 
     def initialize(*)
@@ -589,11 +550,11 @@ class VCS
 
     def branch_beginning(url)
       cmd_read(%W[ #{COMMAND} log -n1 --format=format:%H
-                   --author=matz --committer=matz --grep=has\ started
+                   --author=matz --committer=matz --grep=started\\.$
                    #{url.to_str} -- version.h include/ruby/version.h])
     end
 
-    def export_changelog(url, from, to, path)
+    def export_changelog(url = '@', from = nil, to = nil, _path = nil, path: _path, base_url: nil)
       svn = nil
       from, to = [from, to].map do |rev|
         rev or next
@@ -605,13 +566,16 @@ class VCS
         end
         rev unless rev.empty?
       end
-      unless /./.match(from) or /./.match(from = branch_beginning(url))
+      unless (from && /./.match(from)) or ((from = branch_beginning(url)) && /./.match(from))
         warn "no starting commit found", uplevel: 1
         from = nil
       end
-      unless svn or system(*%W"#{COMMAND} fetch origin refs/notes/commits:refs/notes/commits",
+      if svn or system(*%W"#{COMMAND} fetch origin refs/notes/commits:refs/notes/commits",
                            chdir: @srcdir, exception: false)
-        abort "Could not fetch notes/commits tree"
+        system(*%W"#{COMMAND} fetch origin refs/notes/log-fix:refs/notes/log-fix",
+               chdir: @srcdir, exception: false)
+      else
+        warn "Could not fetch notes/commits tree", uplevel: 1
       end
       to ||= url.to_str
       if from
@@ -619,29 +583,97 @@ class VCS
       else
         arg = ["--since=25 Dec 00:00:00", to]
       end
-      if svn
-        format_changelog_as_svn(path, arg)
+      writer =
+        if svn
+          format_changelog_as_svn(path, arg)
+        else
+          if base_url == true
+            remote, = upstream
+            if remote &&= cmd_read(env, %W[#{COMMAND} remote get-url --no-push #{remote}])
+              remote.chomp!
+              # hack to redirect git.r-l.o to github
+              remote.sub!(/\Agit@git\.ruby-lang\.org:/, 'git@github.com:ruby/')
+              remote.sub!(/\Agit@(.*?):(.*?)(?:\.git)?\z/, 'https://\1/\2/commit/')
+            end
+            base_url = remote
+          end
+          format_changelog(path, arg, base_url)
+        end
+      if !path or path == '-'
+        writer[$stdout]
       else
-        format_changelog(path, arg)
+        File.open(path, 'wb', &writer)
       end
     end
 
-    def format_changelog(path, arg)
+    LOG_FIX_REGEXP_SEPARATORS = '/!:;|,#%&'
+
+    def format_changelog(path, arg, base_url = nil)
       env = {'TZ' => 'JST-9', 'LANG' => 'C', 'LC_ALL' => 'C'}
-      cmd = %W"#{COMMAND} log --format=medium --notes=commits --topo-order --no-merges"
+      cmd = %W"#{COMMAND} log --format=fuller --notes=commits --notes=log-fix --topo-order --no-merges"
       date = "--date=iso-local"
-      unless system(env, *cmd, date, chdir: @srcdir, out: NullDevice, exception: false)
+      unless system(env, *cmd, date, "-1", chdir: @srcdir, out: NullDevice, exception: false)
         date = "--date=iso"
       end
       cmd << date
       cmd.concat(arg)
-      system(env, *cmd, chdir: @srcdir, out: path)
+      proc do |w|
+        w.print "-*- coding: utf-8 -*-\n\n"
+        w.print "base-url = #{base_url}\n\n" if base_url
+        cmd_pipe(env, cmd, chdir: @srcdir) do |r|
+          while s = r.gets("\ncommit ")
+            h, s = s.split(/^$/, 2)
+            h.gsub!(/^(?:(?:Author|Commit)(?:Date)?|Date): /, '  \&')
+            if s.sub!(/\nNotes \(log-fix\):\n((?: +.*\n)+)/, '')
+              fix = $1
+              s = s.lines
+              fix.each_line do |x|
+                case x
+                when %r[^ +(\d+)s([#{LOG_FIX_REGEXP_SEPARATORS}])(.+)\2(.*)\2]o
+                  n = $1.to_i
+                  wrong = $3
+                  correct = $4
+                  begin
+                    s[n][wrong] = correct
+                  rescue IndexError
+                    message = ["format_changelog failed to replace #{wrong.dump} with #{correct.dump} at #$1\n"]
+                    from = [1, n-2].max
+                    to = [s.size-1, n+2].min
+                    s.each_with_index do |e, i|
+                      next if i < from
+                      break if to < i
+                      message << "#{i}:#{e}"
+                    end
+                    raise message.join('')
+                  end
+                when %r[^( +)(\d+)i([#{LOG_FIX_REGEXP_SEPARATORS}])(.*)\3]o
+                  s[$2.to_i, 0] = "#{$1}#{$4}\n"
+                when %r[^ +(\d+)(?:,(\d+))?d]
+                  n = $1.to_i
+                  e = $2
+                  s[n..(e ? e.to_i : n)] = []
+                end
+              end
+              s = s.join('')
+            end
+
+            if %r[^ +(https://github\.com/[^/]+/[^/]+/)commit/\h+\n(?=(?: +\n(?i: +Co-authored-by: .*\n)+)?(?:\n|\Z))] =~ s
+              issue = "#{$1}pull/"
+              s.gsub!(/\b[Ff]ix(?:e[sd])? \K#(?=\d+)/) {issue}
+            end
+
+            s.gsub!(/ +\n/, "\n")
+            s.sub!(/^Notes:/, '  \&')
+            w.print h, s
+          end
+        end
+      end
     end
 
     def format_changelog_as_svn(path, arg)
       cmd = %W"#{COMMAND} log --topo-order --no-notes -z --format=%an%n%at%n%B"
       cmd.concat(arg)
-      open(path, 'w') do |w|
+      proc do |w|
         sep = "-"*72 + "\n"
         w.print sep
         cmd_pipe(cmd) do |r|
@@ -707,7 +739,7 @@ class VCS
 
       commits = cmd_read([COMMAND, "log", "--reverse", "--format=%H %ae %ce", "#{com}..@"], "rb").split("\n")
       commits.each_with_index do |l, i|
-        r, a, c = l.split
+        r, a, c = l.split(' ')
         dcommit = [COMMAND, "svn", "dcommit"]
         dcommit.insert(-2, "-n") if dryrun
         dcommit << "--add-author-from" unless a == c
